;nyquist plug-in
;version 1
;type process
;name "Broadcast Limiter IV..."
;action "Broadcast Limiter IV..."

;control threshold "Threshold" real "linear" 0.5 0.05 1.0
;control knee "Knee" real "dB" 0.5 0.5 6.0
;control limit "Output Level" real "dB" -0.5 -6.0 0.0

(defmacro variable-limit (var min max)
  `(cond ((< ,var ,min)
          (format t "Setting ~a to ~a~%" ',var ,min)
          (setq ,var ,min))
         ((> ,var ,max)
          (format t "Setting ~a to ~a~%" ',var ,max)
          (setq ,var ,max))))

(variable-limit threshold 0.05 1.0)
(variable-limit knee 0.5 20.0)
(variable-limit limit -20.0 0.0)

(let* ((sound-srate   (truncate *sound-srate*))
       (step-size     (/ 1.0 *sound-srate*))
       (amplification (/ 1.0 threshold))
       (shape-array   (make-array (1+ (* sound-srate 2))))
       (knee-lower    (db-to-linear (- limit (/ knee 2))))
       (knee-upper    (db-to-linear (+ limit (/ knee 2))))
       (knee-range    (- knee-upper knee-lower))
       (linear-limit  (db-to-linear limit))
       (rsqrt-2       (/ 1.0 (sqrt 2.0)))  ; 1/sqrt(2)
       (db            (linear-to-db (/ (- 1.0 rsqrt-2) rsqrt-2)))
       (db-range      (/ db knee-range)))

  (defun coord (value)
    (cond ((< value knee-lower) value)
          ((> value knee-upper) linear-limit)
          (t (let ((signal (- value knee-lower)))
               (+ knee-lower (* (db-to-linear (* db-range signal))
                                signal))))))

  ;; transformation data for positive halfwave
  (dotimes (i (1+ sound-srate))
    (setf (aref shape-array (+ sound-srate i))
          (coord (* i step-size amplification))))

  ;; mirror transformation data to negative halfwave
  (dotimes (i sound-srate)
    (setf (aref shape-array (- sound-srate i 1))
          (- (aref shape-array (+ sound-srate i 1)))))

  ;; audio function
  (shape s (snd-from-array 0.0 *sound-srate* shape-array) 1.0))